local M = {}

local running = true
local scenario = nil
local speed = 0
local crash = 0

local helper = require('scenario/scenariohelper')

local function reset()
    running = false
	crash = 0
end

local function fail(reason)
    scenarios.finish({failed = reason})
    reset()
end

local function success(speed)
    local result = {msg = 'Total damage done: $'..string.format("%0.2f", speed)..''}
    scenarios.finish(result)
    reset()
end

local function onRaceStart()
    reset()

    helper.init()
    helper.setAiMode('aiCar1', 'manual')
    helper.setAiAggression('aiCar1', 0.15)
    helper.setAiTarget('aiCar1', 'PikeSB_BH')
    helper.trackVehicle('aiCar1', 'aiCarA')
	
	helper.setAiMode('aiCar2', 'manual')
    helper.setAiAggression('aiCar2', 1)
    helper.setAiTarget('aiCar2', '')
    helper.trackVehicle('aiCar2', 'aiCarB')
	
	local car3 = helper.getVehicleByName('aiCar3')
    car3:queueLuaCommand('scenario.freeze(1)')
	helper.trackVehicle('aiCar3', 'aiCarC')

    helper.trackVehicle('scenario_player0', 'playerCar')
	
	helper.queueLuaCommandByName('aiCar3', 'electrics.toggle_lightbar_signal()')

    running = true
end

local function onRaceTick(raceTickTime)
    if not running then
        return
    end

    helper.update(raceTickTime)

    local aiCarAData = map.objects[map.objectNames['aiCarA']]
	local aiCarBData = map.objects[map.objectNames['aiCarB']]
	local aiCarCData = map.objects[map.objectNames['aiCarC']]
    local playerCarData = map.objects[map.objectNames['playerCar']]

    if not aiCarAData or not playerCarData or not aiCarBData then
        return
    end

    if aiCarAData.damage > 0 and speed == 0 then
        speed = playerCarData.vel:length()
    end
	
	--if playerCarData.damage >= 50.0 and crash == 1 then
	--	helper.queueLuaCommandByName('scenario_player0', 'bullettime.set(2)')
    --end
	
	local player = scenetree.findObject('scenario_player0')
	local playerPos = player:getPosition()
	local playerY = playerPos.y
	
	if playerY <= -115 then
		helper.queueLuaCommandByName('scenario_player0', 'bullettime.set(4)')
	end
	
	if playerCarData.damage >= 75000 then
		fail("You crashed your car!")
	end

    if aiCarAData.damage >= 50.0 then
		helper.setAiMode('aiCar1', 'stop')
		crash = crash + 1
    end
	
	if aiCarBData.damage >= 50.0 then
		helper.setAiMode('aiCar2', 'stop')
		crash = crash + 1
    end
	
	if aiCarCData.damage >= 50.0 then
		crash = crash + 1
    end
	
	if scenario.timer > 14 then
		helper.setAiTarget('aiCar2', 'PikeNB_BB')
	end

    if scenario.timer > 42 then
		local playerCar = helper.getVehicleByName('scenario_player0')
		if crash == 0 then
			fail("You lost!")
		elseif crash >= 1 then
			trafficDamage = ((aiCarAData.damage + aiCarBData.damage + aiCarCData.damage) * 0.1) * crash
			playerDamage = playerCarData.damage * 0.5
			totalDamage = trafficDamage - playerDamage
			if totalDamage < 2000000 then
				fail("You didn't do enough damage!")
			elseif totalDamage >= 2000000 then
				success(totalDamage)
			end
		end
    end
end

local function onScenarioChange(sc)
    scenario = sc
end

M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
M.onScenarioChange = onScenarioChange

return M