local M = {}

local running = true
local scenario = nil
local speed = 0
local crash = 0

local helper = require('scenario/scenariohelper')

local function reset()
    running = false
	crash = 0
end

local function fail(reason)
    scenarios.finish({failed = reason})
    reset()
end

local function success(speed)
    local result = {msg = 'Target damage done: $'..string.format("%0.2f", speed)..''}
    scenarios.finish(result)
    reset()
end

local function onRaceStart()
    reset()

    helper.init()
    helper.setAiMode('aiCar1', 'manual')
    helper.setAiAggression('aiCar1', 1)
    helper.setAiTarget('aiCar1', 'HawthorneNB_AD')
    helper.trackVehicle('aiCar1', 'aiCarA')
	
    helper.trackVehicle('target', 'targetCar')
	
	local targetCar = helper.getVehicleByName('target')
    targetCar:queueLuaCommand('scenario.freeze(1)')

    helper.trackVehicle('scenario_player0', 'playerCar')

    running = true
end

local function onRaceTick(raceTickTime)
    if not running then
        return
    end

    helper.update(raceTickTime)

    local aiCarAData = map.objects[map.objectNames['aiCarA']]
	local targetData = map.objects[map.objectNames['targetCar']]
    local playerCarData = map.objects[map.objectNames['playerCar']]

    if not aiCarAData or not playerCarData then
        return
    end

    if aiCarAData.damage > 0 and speed == 0 then
        speed = playerCarData.vel:length()
    end
	
	if playerCarData.damage >= 50.0 and crash >= 1 then
		helper.queueLuaCommandByName('scenario_player0', 'bullettime.set(2)')
		local playerCar = helper.getVehicleByName('scenario_player0')
        playerCar:queueLuaCommand('scenario.freeze(1)')
    end

    if aiCarAData.damage >= 1000.0 then
		helper.setAiMode('aiCar1', 'stop')
		crash = 1
    end
	
	if targetData.damage >= 500.0 then
		crash = 1
    end
	
    if scenario.timer > 19 then
		local playerCar = helper.getVehicleByName('scenario_player0')
        playerCar:queueLuaCommand('scenario.freeze(0)')
		if crash >= 1 then
			targetDamage = targetData.damage
			trafficDamage = (aiCarAData.damage) * 4
			totalDamage = targetDamage - trafficDamage
			if targetDamage >= 15000 then
				if totalDamage >= 15000 then
					success(targetDamage)
				else
					fail("You did too much damage to the traffic!")
				end
			else
				fail("You didn't do enough damage to the target!")
			end
		else
			fail("You didn't hit anything!")
		end
    end
end

local function onScenarioChange(sc)
    scenario = sc
end

M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
M.onScenarioChange = onScenarioChange

return M