local M = {}

local running = true
local scenario = nil
local speed = 0
local crash = 0

local helper = require('scenario/scenariohelper')

local function reset()
    running = false
	crash = 0
end

local function fail(reason)
    scenarios.finish({failed = reason})
    reset()
end

local function success(speed)
    local result = {msg = 'Total damage done: $'..string.format("%0.2f", speed)..''}
    scenarios.finish(result)
    reset()
end

local function onRaceStart()
    reset()

    helper.init()
    helper.setAiMode('aiCar1', 'manual')
    helper.setAiAggression('aiCar1', 0.15)
    helper.setAiTarget('aiCar1', 'LombardEB_AA')
    helper.trackVehicle('aiCar1', 'aiCarA')
	
	helper.setAiMode('aiCar2', 'manual')
    helper.setAiAggression('aiCar2', 0.6)
    helper.setAiTarget('aiCar2', 'LombardWB_AH')
    helper.trackVehicle('aiCar2', 'aiCarB')

    helper.trackVehicle('scenario_player0', 'playerCar')

    running = true
end

local function onRaceTick(raceTickTime)
    if not running then
        return
    end

    helper.update(raceTickTime)

    local aiCarAData = map.objects[map.objectNames['aiCarA']]
	local aiCarBData = map.objects[map.objectNames['aiCarB']]
    local playerCarData = map.objects[map.objectNames['playerCar']]

    if not aiCarAData or not playerCarData or not aiCarBData then
        return
    end

    if aiCarAData.damage > 0 and speed == 0 then
        speed = playerCarData.vel:length()
    end
	
	if playerCarData.damage >= 50.0 and crash == 1 then
		helper.queueLuaCommandByName('scenario_player0', 'bullettime.set(2)')
		local playerCar = helper.getVehicleByName('scenario_player0')
        playerCar:queueLuaCommand('scenario.freeze(1)')
    end

    if aiCarAData.damage >= 50.0 then
		helper.setAiMode('aiCar1', 'stop')
		crash = crash + 1
    end
	
	if aiCarBData.damage >= 50.0 then
		helper.setAiMode('aiCar2', 'stop')
		crash = crash + 1
    end

    if scenario.timer > 13 then
		local playerCar = helper.getVehicleByName('scenario_player0')
        playerCar:queueLuaCommand('scenario.freeze(0)')
		if crash == 0 then
			fail("You lost!")
		elseif crash >= 1 then
			totalDamage = ((aiCarAData.damage + aiCarBData.damage +  playerCarData.damage) * 0.1) * crash
			if totalDamage < 10000 then
				fail("You didn't do enough damage!")
			elseif totalDamage >= 10000 then
				success(totalDamage)
			end
		end
    end
end

local function onScenarioChange(sc)
    scenario = sc
end

M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
M.onScenarioChange = onScenarioChange

return M