local M = {}

local running = true
local scenario = nil
local speed = 0
local crash = 0

local helper = require('scenario/scenariohelper')

local function reset()
    running = false
	crash = 0
	helper.queueLuaCommandByName('scenario_player0', 'bullettime.set(2.2)')
end

local function fail(reason)
    scenarios.finish({failed = reason})
    reset()
end

local function success(speed)
    local result = {msg = 'Total damage done: $'..string.format("%0.2f", speed)..''}
    scenarios.finish(result)
    reset()
end

local function onRaceStart()
    reset()

    helper.init()
    helper.setAiMode('aiCar1', 'manual')
    helper.setAiAggression('aiCar1', 3.0)
    helper.setAiTarget('aiCar1', 'HawthorneNB_AD')
    helper.trackVehicle('aiCar1', 'aiCarA')
	
	helper.setAiMode('aiCar2', 'manual')
    helper.setAiAggression('aiCar2', 3.9)
    helper.setAiTarget('aiCar2', 'LombardEB_AA')
    helper.trackVehicle('aiCar2', 'aiCarB')
	
	helper.setAiMode('aiCar3', 'manual')
    helper.setAiAggression('aiCar3', 3.0)
    helper.setAiTarget('aiCar3', 'HawthorneSB_AKL')
    helper.trackVehicle('aiCar3', 'aiCarC')
	
	helper.setAiMode('aiCar4', 'manual')
    helper.setAiAggression('aiCar4', 3.0)
    helper.setAiTarget('aiCar4', 'HawthorneSB_AKL')
    helper.trackVehicle('aiCar4', 'aiCarD')
	
	helper.setAiMode('aiCar5', 'manual')
    helper.setAiAggression('aiCar5', 3.0)
    helper.setAiTarget('aiCar5', 'HawthorneSB_AKL')
    helper.trackVehicle('aiCar5', 'aiCarE')
	
	helper.trackVehicle('aiCar6', 'aiCarF')
	helper.queueLuaCommandByName('aiCar6', 'electrics.toggle_lightbar_signal()')

    helper.trackVehicle('scenario_player0', 'playerCar')

    running = true
end

local function onRaceTick(raceTickTime)
    if not running then
        return
    end

    helper.update(raceTickTime)

    local aiCarAData = map.objects[map.objectNames['aiCarA']]
	local aiCarBData = map.objects[map.objectNames['aiCarB']]
	local aiCarCData = map.objects[map.objectNames['aiCarC']]
	local aiCarDData = map.objects[map.objectNames['aiCarD']]
	local aiCarEData = map.objects[map.objectNames['aiCarE']]
	local aiCarFData = map.objects[map.objectNames['aiCarF']]
    local playerCarData = map.objects[map.objectNames['playerCar']]

    if not aiCarAData or not playerCarData or not aiCarBData then
        return
    end

    if aiCarAData.damage > 0 and speed == 0 then
        speed = playerCarData.vel:length()
    end
	
	if playerCarData.damage >= 50.0 and crash == 1 then
		helper.queueLuaCommandByName('scenario_player0', 'bullettime.set(2.8)')
		local playerCar = helper.getVehicleByName('scenario_player0')
        playerCar:queueLuaCommand('scenario.freeze(1)')
    end

    if aiCarAData.damage >= 500.0 then
		helper.setAiMode('aiCar1', 'stop')
		crash = crash + 1
    end
	
	if aiCarBData.damage >= 500.0 then
		helper.setAiMode('aiCar2', 'stop')
		crash = crash + 1
    end
	
	if aiCarCData.damage >= 500.0 then
		helper.setAiMode('aiCar3', 'stop')
		crash = crash + 1
    end
	
	if aiCarDData.damage >= 500.0 then
		helper.setAiMode('aiCar4', 'stop')
		crash = crash + 1
    end
	
	if aiCarEData.damage >= 500.0 then
		helper.setAiMode('aiCar5', 'stop')
		crash = crash + 1
    end
	
	if aiCarEData.damage >= 500.0 then
		crash = crash + 1
    end

    if scenario.timer > 40 then
		local playerCar = helper.getVehicleByName('scenario_player0')
        playerCar:queueLuaCommand('scenario.freeze(0)')
		if crash == 0 then
			fail("You didn't hit anything!")
		elseif crash >= 1 then
			totalDamage = ((aiCarAData.damage + aiCarBData.damage + aiCarCData.damage + aiCarDData.damage + aiCarEData.damage + aiCarFData.damage + playerCarData.damage) * 0.1) * crash
			if totalDamage < 4000000 then
				fail("You didn't do enough damage!")
			elseif totalDamage >= 4000000 then
				success(totalDamage)
			end
		end
    end
end

local function onScenarioChange(sc)
    scenario = sc
end

M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
M.onScenarioChange = onScenarioChange

return M